@@ -9,7 +9,6 @@ import 'dart:_interceptors';
 import 'dart:_js_helper'
     show
         assertUnreachable,
-        boolConversionCheck,
         checkInt,
         Closure,
         ConstantMap,
@@ -23,7 +22,7 @@ import 'dart:_js_helper'
         quoteStringForRegExp,
         getTraceFromException,
         RuntimeError,
-        wrapException,
+        initializeExceptionWrapper,
         wrapZoneUnaryCallback,
         TrustedGetRuntimeType;
 
@@ -42,7 +41,7 @@ String _symbolToString(Symbol symbol) =>
 
 Map<String, dynamic>? _symbolMapToStringMap(Map<Symbol, dynamic>? map) {
   if (map == null) return null;
-  var result = new Map<String, dynamic>();
+  var result = Map<String, dynamic>();
   map.forEach((Symbol key, value) {
     result[_symbolToString(key)] = value;
   });
@@ -66,7 +65,7 @@ class Object {
 
   @patch
   dynamic noSuchMethod(Invocation invocation) {
-    throw new NoSuchMethodError.withInvocation(this, invocation);
+    throw NoSuchMethodError.withInvocation(this, invocation);
   }
 
   @patch
@@ -83,14 +82,18 @@ class Null {
 @patch
 class Function {
   @patch
-  static apply(Function function, List<dynamic>? positionalArguments,
-      [Map<Symbol, dynamic>? namedArguments]) {
+  static apply(
+    Function function,
+    List<dynamic>? positionalArguments, [
+    Map<Symbol, dynamic>? namedArguments,
+  ]) {
     return Primitives.applyFunction(
-        function,
-        positionalArguments,
-        // Use this form so that if namedArguments is always null, we can
-        // tree-shake _symbolMapToStringMap.
-        namedArguments == null ? null : _symbolMapToStringMap(namedArguments));
+      function,
+      positionalArguments,
+      // Use this form so that if namedArguments is always null, we can
+      // tree-shake _symbolMapToStringMap.
+      namedArguments == null ? null : _symbolMapToStringMap(namedArguments),
+    );
   }
 }
 
@@ -132,8 +135,11 @@ class Expando<T extends Object> {
   }
 
   static Never _badExpandoKey(object) {
-    throw ArgumentError.value(object, 'object',
-        "Expandos are not allowed on strings, numbers, bools, records or null");
+    throw ArgumentError.value(
+      object,
+      'object',
+      "Expandos are not allowed on strings, numbers, bools, records or null",
+    );
   }
 }
 
@@ -151,7 +157,7 @@ class _WeakReferenceWrapper<T extends Object> implements WeakReference<T> {
   final Object _weakRef;
 
   _WeakReferenceWrapper(T object)
-      : _weakRef = JS('', 'new #(#)', _weakRefConstructor, object);
+    : _weakRef = JS('', 'new #(#)', _weakRefConstructor, object);
 
   T? get target => JS('', '#.deref()', _weakRef);
 
@@ -196,10 +202,14 @@ class _FinalizationRegistryWrapper<T> implements Finalizer<T> {
   final Object? _registry;
 
   _FinalizationRegistryWrapper(void Function(T) callback)
-      : _registry = _finalizationRegistryConstructor == null
-            ? null
-            : JS('', 'new #(#)', _finalizationRegistryConstructor,
-                convertDartClosureToJS(wrapZoneUnaryCallback(callback), 1));
+    : _registry = _finalizationRegistryConstructor == null
+          ? null
+          : JS(
+              '',
+              'new #(#)',
+              _finalizationRegistryConstructor,
+              convertDartClosureToJS(wrapZoneUnaryCallback(callback), 1),
+            );
 
   void attach(Object value, T token, {Object? detach}) {
     if (_registry != null) {
@@ -219,19 +229,22 @@ class _FinalizationRegistryWrapper<T> implements Finalizer<T> {
 
   static final Object? _finalizationRegistryConstructor =
       JS('', 'typeof FinalizationRegistry == "function"')
-          ? JS('', 'FinalizationRegistry')
-          : null;
+      ? JS('', 'FinalizationRegistry')
+      : null;
 }
 
 @patch
 class int {
   @patch
-  static int parse(String source,
-      {int? radix, @deprecated int onError(String source)?}) {
+  static int parse(
+    String source, {
+    int? radix,
+    @deprecated int onError(String source)?,
+  }) {
     int? value = tryParse(source, radix: radix);
     if (value != null) return value;
     if (onError != null) return onError(source);
-    throw new FormatException(source);
+    throw FormatException(source);
   }
 
   @patch
@@ -243,12 +256,14 @@ class int {
 @patch
 class double {
   @patch
-  static double parse(String source,
-      [@deprecated double onError(String source)?]) {
+  static double parse(
+    String source, [
+    @deprecated double onError(String source)?,
+  ]) {
     double? value = tryParse(source);
     if (value != null) return value;
     if (onError != null) return onError(source);
-    throw new FormatException('Invalid double', source);
+    throw FormatException('Invalid double', source);
   }
 
   @patch
@@ -273,162 +288,14 @@ class Error {
   StackTrace? get stackTrace => Primitives.extractStackTrace(this);
 
   @patch
+  @pragma('dart2js:never-inline')
   static Never _throw(Object error, StackTrace stackTrace) {
-    error = wrapException(error);
+    error = initializeExceptionWrapper(error, JS('', 'new Error()'));
     JS('void', '#.stack = #', error, stackTrace.toString());
-    JS('', 'throw #', error);
-    throw "unreachable";
+    JS<Never>('', 'throw #', error);
   }
 }
 
-// Patch for DateTime implementation.
-@patch
-class DateTime {
-  @patch
-  DateTime.fromMillisecondsSinceEpoch(int millisecondsSinceEpoch,
-      {bool isUtc = false})
-      // `0 + millisecondsSinceEpoch` forces the inferred result to be non-null.
-      : this._withValue(0 + millisecondsSinceEpoch, isUtc: isUtc);
-
-  @patch
-  DateTime.fromMicrosecondsSinceEpoch(int microsecondsSinceEpoch,
-      {bool isUtc = false})
-      : this._withValue(
-            _microsecondInRoundedMilliseconds(microsecondsSinceEpoch),
-            isUtc: isUtc);
-
-  @patch
-  DateTime._internal(int year, int month, int day, int hour, int minute,
-      int second, int millisecond, int microsecond, bool isUtc)
-      // checkBool is manually inlined here because dart2js doesn't inline it
-      // and [isUtc] is usually a constant.
-      : this.isUtc = isUtc is bool
-            ? isUtc
-            : throw new ArgumentError.value(isUtc, 'isUtc'),
-        _value = checkInt(Primitives.valueFromDecomposedDate(
-            year,
-            month,
-            day,
-            hour,
-            minute,
-            second,
-            millisecond + _microsecondInRoundedMilliseconds(microsecond),
-            isUtc));
-
-  @patch
-  DateTime._now()
-      : isUtc = false,
-        _value = Primitives.dateNow();
-
-  @patch
-  DateTime._nowUtc()
-      : isUtc = true,
-        _value = Primitives.dateNow();
-
-  /// Rounds the given [microsecond] to the nearest milliseconds value.
-  ///
-  /// For example, invoked with argument `2600` returns `3`.
-  static int _microsecondInRoundedMilliseconds(int microsecond) {
-    return (microsecond / 1000).round();
-  }
-
-  @patch
-  static int? _brokenDownDateToValue(int year, int month, int day, int hour,
-      int minute, int second, int millisecond, int microsecond, bool isUtc) {
-    return Primitives.valueFromDecomposedDate(
-        year,
-        month,
-        day,
-        hour,
-        minute,
-        second,
-        millisecond + _microsecondInRoundedMilliseconds(microsecond),
-        isUtc);
-  }
-
-  @patch
-  String get timeZoneName {
-    if (isUtc) return "UTC";
-    return Primitives.getTimeZoneName(this);
-  }
-
-  @patch
-  Duration get timeZoneOffset {
-    if (isUtc) return new Duration();
-    return new Duration(minutes: Primitives.getTimeZoneOffsetInMinutes(this));
-  }
-
-  @patch
-  DateTime add(Duration duration) {
-    return new DateTime._withValue(_value + duration.inMilliseconds,
-        isUtc: isUtc);
-  }
-
-  @patch
-  DateTime subtract(Duration duration) {
-    return new DateTime._withValue(_value - duration.inMilliseconds,
-        isUtc: isUtc);
-  }
-
-  @patch
-  Duration difference(DateTime other) {
-    return new Duration(milliseconds: _value - other.millisecondsSinceEpoch);
-  }
-
-  @patch
-  int get millisecondsSinceEpoch => _value;
-
-  @patch
-  int get microsecondsSinceEpoch => 1000 * _value;
-
-  @patch
-  int get year => Primitives.getYear(this);
-
-  @patch
-  int get month => Primitives.getMonth(this);
-
-  @patch
-  int get day => Primitives.getDay(this);
-
-  @patch
-  int get hour => Primitives.getHours(this);
-
-  @patch
-  int get minute => Primitives.getMinutes(this);
-
-  @patch
-  int get second => Primitives.getSeconds(this);
-
-  @patch
-  int get millisecond => Primitives.getMilliseconds(this);
-
-  @patch
-  int get microsecond => 0;
-
-  @patch
-  int get weekday => Primitives.getWeekday(this);
-
-  @patch
-  bool operator ==(Object other) =>
-      other is DateTime &&
-      _value == other.millisecondsSinceEpoch &&
-      isUtc == other.isUtc;
-
-  @patch
-  bool isBefore(DateTime other) => _value < other.millisecondsSinceEpoch;
-
-  @patch
-  bool isAfter(DateTime other) => _value > other.millisecondsSinceEpoch;
-
-  @patch
-  bool isAtSameMomentAs(DateTime other) =>
-      _value == other.millisecondsSinceEpoch;
-
-  @patch
-  int compareTo(DateTime other) =>
-      _value.compareTo(other.millisecondsSinceEpoch);
-}
-
 // Patch for Stopwatch implementation.
 @patch
 class Stopwatch {
@@ -464,8 +331,8 @@ class List<E> {
   @patch
   factory List.filled(int length, E fill, {bool growable = false}) {
     var result = growable
-        ? new JSArray<E>.growable(length)
-        : new JSArray<E>.fixed(length);
+        ? JSArray<E>.growable(length)
+        : JSArray<E>.fixed(length);
     if (length != 0 && fill != null) {
       // TODO(sra): Consider using `Array.fill`.
       for (int i = 0; i < result.length; i++) {
@@ -479,7 +346,7 @@ class List<E> {
 
   @patch
   factory List.empty({bool growable = false}) {
-    return growable ? new JSArray<E>.growable(0) : new JSArray<E>.fixed(0);
+    return growable ? JSArray<E>.growable(0) : JSArray<E>.fixed(0);
   }
 
   @patch
@@ -493,19 +360,13 @@ class List<E> {
   }
 
   @patch
-  factory List.of(Iterable<E> elements, {bool growable = true}) {
-    if (growable == true) return List._of(elements);
-    if (growable == false) return List._fixedOf(elements);
-
-    // [growable] may be `null` in legacy mode. Fail with the same error as if
-    // [growable] was used in a condition position in spec mode.
-    boolConversionCheck(growable);
-    assertUnreachable();
-  }
+  factory List.of(Iterable<E> elements, {bool growable = true}) =>
+      growable ? List._of(elements) : List._fixedOf(elements);
 
   factory List._ofArray(Iterable<E> elements) {
     return JSArray<E>.markGrowable(
-        JS('effects:none;depends:no-static', '#.slice(0)', elements));
+      JS('effects:none;depends:no-static', '#.slice(0)', elements),
+    );
   }
 
   factory List._of(Iterable<E> elements) {
@@ -524,11 +385,14 @@ class List<E> {
   }
 
   @patch
-  factory List.generate(int length, E generator(int index),
-      {bool growable = true}) {
+  factory List.generate(
+    int length,
+    E generator(int index), {
+    bool growable = true,
+  }) {
     final result = growable
-        ? new JSArray<E>.growable(length)
-        : new JSArray<E>.fixed(length);
+        ? JSArray<E>.growable(length)
+        : JSArray<E>.fixed(length);
     for (int i = 0; i < length; i++) {
       result[i] = generator(i);
     }
@@ -554,8 +418,11 @@ class Map<K, V> {
 @patch
 class String {
   @patch
-  factory String.fromCharCodes(Iterable<int> charCodes,
-      [int start = 0, int? end]) {
+  factory String.fromCharCodes(
+    Iterable<int> charCodes, [
+    int start = 0,
+    int? end,
+  ]) {
     RangeError.checkNotNegative(start, "start");
     if (end != null) {
       var maxLength = end - start;
@@ -599,7 +466,10 @@ class String {
   }
 
   static String _stringFromUint8List(
-      NativeUint8List charCodes, int start, int? endOrNull) {
+    NativeUint8List charCodes,
+    int start,
+    int? endOrNull,
+  ) {
     int len = charCodes.length;
     if (start >= len) return "";
     int end = (endOrNull == null || endOrNull > len) ? len : endOrNull;
@@ -607,7 +477,10 @@ class String {
   }
 
   static String _stringFromIterable(
-      Iterable<int> charCodes, int start, int? end) {
+    Iterable<int> charCodes,
+    int start,
+    int? end,
+  ) {
     if (end != null) charCodes = charCodes.take(end);
     if (start > 0) charCodes = charCodes.skip(start);
     return Primitives.stringFromCharCodes(List<int>.of(charCodes));
@@ -634,16 +507,19 @@ class bool {
 class RegExp {
   @pragma('dart2js:noInline')
   @patch
-  factory RegExp(String source,
-          {bool multiLine = false,
-          bool caseSensitive = true,
-          bool unicode = false,
-          bool dotAll = false}) =>
-      new JSSyntaxRegExp(source,
-          multiLine: multiLine,
-          caseSensitive: caseSensitive,
-          unicode: unicode,
-          dotAll: dotAll);
+  factory RegExp(
+    String source, {
+    bool multiLine = false,
+    bool caseSensitive = true,
+    bool unicode = false,
+    bool dotAll = false,
+  }) => JSSyntaxRegExp(
+    source,
+    multiLine: multiLine,
+    caseSensitive: caseSensitive,
+    unicode: unicode,
+    dotAll: dotAll,
+  );
 
   @patch
   static String escape(String text) => quoteStringForRegExp(text);
@@ -651,7 +527,8 @@ class RegExp {
 
 // Patch for 'identical' function.
 @pragma(
-    'dart2js:noInline') // No inlining since we recognize the call in optimizer.
+  'dart2js:noInline',
+) // No inlining since we recognize the call in optimizer.
 @patch
 bool identical(Object? a, Object? b) {
   return JS('bool', '(# == null ? # == null : # === #)', a, b, a, b);
@@ -674,7 +551,7 @@ class StringBuffer {
 
   @patch
   void writeCharCode(int charCode) {
-    _writeString(new String.fromCharCode(charCode));
+    _writeString(String.fromCharCode(charCode));
   }
 
   @patch
@@ -731,24 +608,40 @@ class NoSuchMethodError {
 
   @patch
   factory NoSuchMethodError.withInvocation(
-          Object? receiver, Invocation invocation) =>
-      NoSuchMethodError._(receiver, invocation.memberName,
-          invocation.positionalArguments, invocation.namedArguments);
+    Object? receiver,
+    Invocation invocation,
+  ) => NoSuchMethodError._(
+    receiver,
+    invocation.memberName,
+    invocation.positionalArguments,
+    invocation.namedArguments,
+  );
 
-  NoSuchMethodError(Object? receiver, Symbol memberName,
-      List? positionalArguments, Map<Symbol, dynamic>? namedArguments,
-      [List? existingArgumentNames = null])
-      : this._(receiver, memberName, positionalArguments, namedArguments,
-            existingArgumentNames);
+  NoSuchMethodError(
+    Object? receiver,
+    Symbol memberName,
+    List? positionalArguments,
+    Map<Symbol, dynamic>? namedArguments, [
+    List? existingArgumentNames = null,
+  ]) : this._(
+         receiver,
+         memberName,
+         positionalArguments,
+         namedArguments,
+         existingArgumentNames,
+       );
 
-  NoSuchMethodError._(Object? receiver, Symbol memberName,
-      List? positionalArguments, Map<Symbol, dynamic>? namedArguments,
-      [List? existingArgumentNames = null])
-      : _receiver = receiver,
-        _memberName = memberName,
-        _arguments = positionalArguments,
-        _namedArguments = namedArguments,
-        _existingArgumentNames = existingArgumentNames;
+  NoSuchMethodError._(
+    Object? receiver,
+    Symbol memberName,
+    List? positionalArguments,
+    Map<Symbol, dynamic>? namedArguments, [
+    List? existingArgumentNames = null,
+  ]) : _receiver = receiver,
+       _memberName = memberName,
+       _arguments = positionalArguments,
+       _namedArguments = namedArguments,
+       _existingArgumentNames = existingArgumentNames;
 
   @patch
   String toString() {
@@ -822,35 +715,43 @@ class _Uri {
   @patch
   static bool get _isWindows => _isWindowsCached;
 
-  static final bool _isWindowsCached = JS(
-      'bool',
-      'typeof process != "undefined" && '
-          'Object.prototype.toString.call(process) == "[object process]" && '
-          'process.platform == "win32"');
+  // Consider the possibility of using Windows behavior if app is
+  // compiled with `--server-mode` and running on Node or a similar platform.
+  static final bool _isWindowsCached =
+      !const bool.fromEnvironment('dart.library.html') &&
+      JS<bool>(
+        'bool',
+        'typeof process != "undefined" && '
+            'Object.prototype.toString.call(process) == "[object process]" && '
+            'process.platform == "win32"',
+      );
 
   // Matches a String that _uriEncodes to itself regardless of the kind of
   // component.  This corresponds to [_unreservedTable], i.e. characters that
   // are not encoded by any encoding table.
-  static final RegExp _needsNoEncoding = new RegExp(r'^[\-\.0-9A-Z_a-z~]*$');
+  static final RegExp _needsNoEncoding = RegExp(r'^[\-\.0-9A-Z_a-z~]*$');
 
   /// This is the internal implementation of JavaScript's encodeURI function.
   /// It encodes all characters in the string [text] except for those
   /// that appear in [canonicalTable], and returns the escaped string.
   @patch
-  static String _uriEncode(List<int> canonicalTable, String text,
-      Encoding encoding, bool spaceToPlus) {
+  static String _uriEncode(
+    int canonicalMask,
+    String text,
+    Encoding encoding,
+    bool spaceToPlus,
+  ) {
     if (identical(encoding, utf8) && _needsNoEncoding.hasMatch(text)) {
       return text;
     }
 
     // Encode the string into bytes then generate an ASCII only string
     // by percent encoding selected bytes.
-    StringBuffer result = new StringBuffer('');
+    StringBuffer result = StringBuffer('');
     var bytes = encoding.encode(text);
     for (int i = 0; i < bytes.length; i++) {
       int byte = bytes[i];
-      if (byte < 128 &&
-          ((canonicalTable[byte >> 4] & (1 << (byte & 0x0f))) != 0)) {
+      if (byte < 128 && ((_charTables.codeUnitAt(byte) & canonicalMask) != 0)) {
         result.writeCharCode(byte);
       } else if (spaceToPlus && byte == _SPACE) {
         result.write('+');
@@ -866,7 +767,8 @@ class _Uri {
 
   @patch
   static String _makeQueryFromParameters(
-      Map<String, dynamic /*String?|Iterable<String>*/ > queryParameters) {
+    Map<String, dynamic /*String?|Iterable<String>*/> queryParameters,
+  ) {
     if (!_useURLSearchParams) {
       return _makeQueryFromParametersDefault(queryParameters);
     }
@@ -915,15 +817,18 @@ class _Uri {
 
     // Handle other cases with one RegExp.
     encoded = JS(
-        '',
-        r'#.replace(/=&|\*|%7E/g, (m) => m === "=&" ? "&" : m === "*" ? "%2A" : "~")',
-        encoded);
+      '',
+      r'#.replace(/=&|\*|%7E/g, (m) => m === "=&" ? "&" : m === "*" ? "%2A" : "~")',
+      encoded,
+    );
 
     return encoded;
   }
 
-  static final bool _useURLSearchParams =
-      JS('bool', 'typeof URLSearchParams == "function"');
+  static final bool _useURLSearchParams = JS(
+    'bool',
+    'typeof URLSearchParams == "function"',
+  );
 }
 
 @patch
@@ -951,8 +856,9 @@ class _DuplicatedFieldInitializerError extends Error {
 /// The signature is hardwired to the kernel nodes generated in the
 /// `Dart2jsTarget` and read in the `KernelSsaGraphBuilder`.
 external Invocation _createInvocationMirror(
-    String memberName,
-    List typeArguments,
-    List positionalArguments,
-    Map<String, dynamic> namedArguments,
-    int kind);
+  String memberName,
+  List typeArguments,
+  List positionalArguments,
+  Map<String, dynamic> namedArguments,
+  int kind,
+);
